

C-Programmierung

praxis

c-stringlib


!!! Bstr_substr.c

/* Bstr_substr.c */

#include "Bstr.h"

/* teilstring von "start" der laenge "len" 
   zurueckgeben, zaehlbeginn bei 0; 
   ist start ausserhalb von "s", NULL zurueck,
   ist start+len ausserhalb, len passend kuerzen.
 */
char *Bstr_substr(const char *s, 
		  int start, 
		  int len)
{
  char *temp;
  size_t i, slen;

  slen = strlen(s);

  if ((start > slen) || (start < 0))
    temp = (char *) 0;
  else
    {
      if ((start + len) > slen)
	len = slen - start;

      temp = Bstr_malloc(len + 1);
      for (i = 0; i < len; i++)
	temp[i] = s[start + i];

      temp[i] = '\0';
    }

  return (temp);
}

@bu = Basis fr BASIC-hnliches: <I>Bstr_substr()<D>
(Listing 1).

!!! Bstr_basic_like.c 

/* Bstr_basic_like.c */

#include "Bstr.h"

char *Bstr_leftstr(const char *s, 
		   int len)
{
  return (Bstr_substr(s, 0, len));
}

char *Bstr_rightstr(const char *s, 
		    int len)
{
  char *t1 = Bstr_reverse(s);
  char *t2 = Bstr_substr(t1, 0, len);
  char *new = Bstr_reverse(t2);
  free(t1);
  free(t2);

  /* also nicht:
  return(Bstr_reverse
	 (Bstr_substr
	  (Bstr_reverse(s),
	   0,
	   len)));
	   */

  return (new);
}

@bu = Nur des Namens wegen: <I>Bstr_leftstr()<D> und
<I>Bstr_rightstr()<D> (Listing 2).


!!! Bstr_fetch_word.c

/* Bstr_fetch_word.c */

#include "Bstr.h"

static const char *def_delim = ".,;:!\"]/()? \t\n\f\r\0";

char *Bstr_fetch_word(int wordnum, 
		      const char *line, 
		      const char *delim)
{

  size_t linelen, wlen, wstart, linepos;
  char *temp;

  if (!delim)
    delim = def_delim;

  linelen = strlen(line);
  wlen = wstart = linepos = 0;

  while (wordnum && linepos < linelen)
    {
      wstart = strspn(line + linepos, delim) + linepos;
      wlen = strcspn(line + wstart, delim);
      linepos = wstart + wlen;
      wordnum--;
    }

  if (wordnum || !wlen)
    temp = (char *) 0;
  else
    temp = Bstr_substr(line, wstart, wlen);

  return (temp);
}

/* das gleiche von hinten */
char *Bstr_fetch_r_word(int wordnum, 
			const char *line, 
			const char *delim)
{
  char *rline = Bstr_reverse(line);
  char *temp = Bstr_fetch_word(wordnum, rline, delim);
  char *rtemp = Bstr_reverse(temp);
  free(rline);
  free(temp);
  return (rtemp);
}

/* Bstr_reverse: string "gespiegelt" zurueckgeben */
char *Bstr_reverse(const char *line)
{
  int i1, i2, slen;
  char *temp;

  if (line == (char *) 0)
    temp = (char *) 0;
  else
    {
      slen = strlen(line);
      temp = Bstr_malloc(slen + 1);

      i1 = slen - 1;

      for (i2 = 0; i2 < slen; i2++)
	temp[i2] = line[i1--];

      temp[i2] = '\0';
    }
  return (temp);
}

float Bstr_getfloat(int pos, 
		    const char *s, 
		    const char *delim)
{
  char *tmp = Bstr_fetch_word(pos, s, delim);
  float r = atof(tmp);
  free(tmp);
  return (r);
}

float Bstr_getrfloat(int pos, 
		     const char *s, 
		     const char *delim)
{
  char *tmp = Bstr_fetch_r_word(pos, s, delim);
  float r = atof(tmp);
  free(tmp);
  return (r);
}

int Bstr_getint(int pos, 
		const char *s, 
		const char *delim)
{
  char *tmp = Bstr_fetch_word(pos, s, delim);
  int r = atoi(tmp);
  free(tmp);
  return (r);
}

int Bstr_getrint(int pos, 
		 const char *s, 
		 const char *delim)
{
  char *tmp = Bstr_fetch_r_word(pos, s, delim);
  int r = atoi(tmp);
  free(tmp);
  return (r);
}

@bu = Ein Wort mit beliebigen Begrenzern holen:
<I>Bstr_fetch_word()<D> und seine Abkmmlinge (Listing 3).

!!! Bstr_crop.c

/* Bstr_crop.c */

#include "Bstr.h"

char *Bstr_crop(const char *word)
{
  size_t i = strlen(word) - 1;
  char *tmp = Bstr_malloc(i + 2);

  while (word[i] == '\n' || 
	 word[i] == '\r' || 
	 word[i] == ' ')
    i--;

  strncpy(tmp, word, ++i);
  tmp[++i] = '\0';

  return (tmp);
}

@bu = Strendes am Ende lschen: <I>Bstr_crop()<D>
(Listing 4).

!!! Bstr_insert.c

/* Bstr_insert.c */

#include "Bstr.h"

char *Bstr_insert(const char *instring, 
		  const char *orgstring, 
		  int pos)
{
  size_t instringlen = strlen(instring);
  size_t orgstringlen = strlen(orgstring);
  char *new = Bstr_malloc(orgstringlen + instringlen + 1);

  pos = pos < 0 ? 0 : pos;
  pos = pos > orgstringlen ? orgstringlen : pos;


  strncpy(new, orgstring, pos);
  strcat(new, instring);
  strcat(new, orgstring + pos);

  return (new);
}

@bu = Einsetzen: <I>Bstr_insert()<D> und ... (Listing 5)

!!! Bstr_replace.c

/* Bstr_replace.c */

#include "Bstr.h"

/* suchw durch replw in line ersetzen, 
   how_much: wie oft, 
   0: so oft wie suchw vorkommt */

char *Bstr_replace(const char *suchw,
		   const char *replw,
		   const char *line,
		   int how_much)
{
  char *sw;
  size_t suchpnt = 0;
  size_t linelen = strlen(line);
  size_t suchwlen = strlen(suchw);
  char *new = "";
  char *o_new;
  char *linetemp = (char *) alloca(linelen + 1);
  if (!linetemp)
    errexit("alloca in Bstr_replace fehlgeschlagen\n");

  how_much = how_much ? how_much : -1;

  strcpy(linetemp, line);

  while ((sw = strstr(linetemp + suchpnt, suchw)) && 
	 how_much--)
    {		     /* suchwort gefunden */
      sw[0] = '\0';  /* teilstring vor suchw bilden */
      o_new = new;
      new = (char *) /* cast away const */ 
	pstr_create(new, linetemp, replw, (char *) 0);
      pstr_delete(o_new);
      linetemp = sw + suchwlen;
    }

  o_new = new;
  new = (char *) pstr_create(new, linetemp);
  pstr_delete(o_new);

  return (new);
}

@bu = ... ersetzen: <I>Bstr_replace()<D> (Listing 6).

!!! Bstr_malloc.c

/* Bstr_malloc.c */

#include "Bstr.h"

char *Bstr_malloc(size_t len)
{
  char *r;

  if (!(r = (char *) malloc(len + 1)))
    errexit("Nicht genuegend Speicher fuer %i Zeichen\n",
	    len + 1);

  r[0] = '\0';
  return (r);
}

@bu = Rckgabeprfung inklusive: <I>Bstr_malloc()<D>
(Listing 7)

!!! Bstr_readlin.c

/* Bstr_readlin.c */

#include "Bstr.h"

/* Bstr_readlin liest max-1 zeichen bzw. bis zu 
   LF oder CR oder CR/LF aus infile in line und 
   gibt line zurueck.
   verhaelt sich wie fgets(), aber behandelt alle 
   arten von zeilenenden (CR, LF, CR/LF) */

char *Bstr_readlin(char *line, 
		   int max, 
		   FILE * infile)
{
  int c = (char) 0;
  int i;

  max--; /* wegen kompatibilitaet zu fgets() */

  for (i = 0; 
       i < max && 
       (c = getc(infile)) != EOF && 
       c != '\n' && 
       c != '\r'; 
       ++i)
    line[i] = c;
  if (c == '\n')
    {
      line[i] = c;
      i++;
    }
  else if (c == '\r')
    {
      line[i] = '\n';
      i++;
      if ((c = getc(infile)) != '\n')
	ungetc(c, infile);
    }
  else if (c == EOF && i == 0)
    {
      line = (char *) 0;
      return (line);
    }

  line[i] = '\0';
  return (line);
}

@bu = Zeilen aus aller Welt: <I>Bstr_readlin()<D>
(Listing 8).

!!! Bstr_updown.c

/* Bstr_updown.c */

#include "Bstr.h"

char *Bstr_downcase(const char *s)
{
  int i = -1;
  char *t = Bstr_malloc(strlen(s) + 1);

  while (s[++i] != '\0')
    t[i] = tolower(s[i]);

  t[i] = '\0';

  return (t);
}

char *Bstr_upcase(const char *s)
{
  int i = -1;
  char *t = Bstr_malloc(strlen(s) + 1);

  while (s[++i] != '\0')
    t[i] = toupper(s[i]);

  t[i] = '\0';

  return (t);
}

@bu = Trivial, aber ntzlich: <I>Bstr_updown()<D>
(Listing 9).

!!! errexit.c

#include "Bstr.h"
#include <varargs.h>

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

void errexit(const char *fmt, ...)
{
  va_list ap;

  fprintf(stderr,"ERREXIT:: ");
  va_start(ap,fmt);
  vfprintf(stderr,fmt,ap);
  va_end(ap);
  exit(EXIT_FAILURE);
}

@bu = Gesprchiger Abbruch: <I>errexit()<D> (Listing 10)

!!! Bstr.h

/* Bstr.h */

#ifndef __Bstr_h__
#define __Bstr_h__

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <ctype.h>
#include <assert.h>
#include "pstr.h"

void errexit(const char *format,...);

char *Bstr_malloc(size_t len);
char *Bstr_downcase(const char *line);
char *Bstr_upcase(const char *line);
char *Bstr_readlin(char *line,
		   int max,
		   FILE * infile);
char *Bstr_insert(const char *instring,
		  const char *orgstring,
		  int pos);
char *Bstr_replace(const char *suchw,
		   const char *replw,
		   const char *line,
		   int how_much);
char *Bstr_fetch_word(int wordnum,
		      const char *line,
		      const char *delim);
char *Bstr_substr(const char *s,
		  int start,
		  int len);
char *Bstr_reverse(const char *line);
char *Bstr_fetch_r_word(int wordnum,
			const char *line,
			const char *delim);
char *Bstr_leftstr(const char *s,
		   int len);
char *Bstr_rightstr(const char *s,
		    int len);
char *Bstr_crop(const char *s);

#define Bstr_midstr(A,B,C) Bstr_substr((A),(B),(C))

int Bstr_getrint(int pos, 
		 const char *s, 
		 const char *delim);

int Bstr_getint(int pos, 
		const char *s, 
		const char *delim);

float Bstr_getrfloat(int pos, 
		     const char *s, 
		     const char *delim);

float Bstr_getfloat(int pos, 
		    const char *s, 
		    const char *delim);

#endif

@bu = Zusammengefat: Header-Datei <I>Bstr.h<D>
(Listing 11)
